#!/usr/bin/env python

import roslib; roslib.load_manifest('harkviz')
import rospy
import threading
import scipy
import time
from hark_msgs.msg import HarkWave
from hark_msgs.msg import HarkWaveVal

from pylab import *
ion()

node_name = 'HarkWave_EachWavePlotter'
topic_name = 'HarkWave'
disp_col_num = int(16000 * 0.3)
disp_interval = 0.1
disp_ymax = 16777216 / 2 / 20
disp_ymin = -1 * 16777216 / 2 / 20
cb_count = 0
zero_ignore = 0
plot_data = []

# Command line arguments

for cnt in  range(len(sys.argv)) :
    if sys.argv[cnt] == '-h' or sys.argv[cnt] == '--help' :
        print '----------------------------------'
        print ''
        print 'USAGE : '
        print 'rosrun harkviz HarkWave_EachWavePlotter.py -c INT -i FLOAT[sec] -t TopicName -n NodeName -z -ymin FLOAT -ymax FLOAT'
        print ''
        print '----------------------------------'
        sys.exit()
    if sys.argv[cnt] == '-c' or sys.argv[cnt] == '--column' :
        try: 
            disp_col_num = int(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -c INT'
            sys.exit()
    if sys.argv[cnt] == '-i' or sys.argv[cnt] == '--interval' :
        try: 
            disp_interval = float(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -i FLOAT'
            sys.exit()
    if sys.argv[cnt] == '-t' or sys.argv[cnt] == '--topic' :
        if '-' == sys.argv[cnt + 1][0] :
            print 'Error : invalid topic name'
            sys.exit()
        topic_name = sys.argv[cnt + 1]
    if sys.argv[cnt] == '-n' or sys.argv[cnt] == '--node' :
        if '-' == sys.argv[cnt + 1][0] :
            print 'Error : invalid topic name'
            sys.exit()
        node_name = sys.argv[cnt + 1]
    if sys.argv[cnt] == '-z' or sys.argv[cnt] == '--zero-ignore' :
        zero_ignore = 1
    if sys.argv[cnt] == '-ymin' or sys.argv[cnt] == '--ymin' :
        try: 
            disp_ymin = float(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -ymin FLOAT'
            sys.exit()
    if sys.argv[cnt] == '-ymax' or sys.argv[cnt] == '--ymax' :
        try: 
            disp_ymax = float(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -ymax FLOAT'
            sys.exit()

# Plotting

def callback(data):        

    global plot_data
    global disp_col_num
    global cb_count
    global zero_ignore
    nzero_flag = 0

    for i in range(len(data.src)):
        for j in range(len(data.src[i].wavedata)):
            if data.src[i].wavedata[j] != 0.0 :
                nzero_flag = 1

    if nzero_flag == 0 and zero_ignore == 1 :
        return    

    for win in range(len(data.src)):
                
        if cb_count == 0:
            new_frame = []
            # for j in range(len(data.src[win].wavedata)):
            for j in range(160):
                new_frame.append(data.src[win].wavedata[j])
            plot_data.append(new_frame)
        else:
            # for j in range(len(data.src[win].wavedata)):
            for j in range(160):
                plot_data[win].append(data.src[win].wavedata[j])
            if len(plot_data[win]) >= disp_col_num :
                plot_data[win] = plot_data[win][len(plot_data[win])-disp_col_num:]
          
    cb_count = cb_count + 1
    # print cb_count

# Plotting

class plotthread(threading.Thread):

    global plot_data
    global disp_interval

    def __init__(self):
        threading.Thread.__init__(self)
        self.setDaemon(True)

    def run(self):
        print "Plotting Start."
        while True :
            for win in range(len(plot_data)):
                pltcnt = 100 * len(plot_data) + 10 + win + 1
                if len(plot_data[win]) >= 1 : 
                    plot(scipy.arange(0,len(plot_data[win]),1), array(plot_data[win]), 'b-')
                    ylim( (disp_ymin, disp_ymax) )
                    draw()    
                    hold(False)
                subplot(pltcnt)
            time.sleep(disp_interval)
                    
# Main

def listener():
    global node_name
    global topic_name
    rospy.init_node(node_name, anonymous=True)
    rospy.Subscriber(topic_name, HarkWave, callback, None, 100)
    print "Plotter ready."
    rospy.spin()

if __name__ == '__main__':
    t = plotthread()
    t.start()
    listener()

